;*==========================================================================;
;*
;*  Copyright (C) Microsoft Corporation.  All Rights Reserved.
;*
;*  File:       d3d9types.h
;*  Content:    Direct3D capabilities include file
;*
;***************************************************************************

ifndef d3d9TYPES_INC
d3d9TYPES_INC equ <1>

D3DXVECTOR3 struct
	x real4 ?
	y real4 ?
	z real4 ?
D3DXVECTOR3 ends


; D3DCOLOR is equivalent to D3DFMT_A8R8G8B8
ifndef D3DCOLOR_DEFINED
D3DCOLOR typedef DWORD 
D3DCOLOR_DEFINED EQU <1>
endif

;// maps unsigned 8 bits/channel to D3DCOLOR
;D3DCOLOR_ARGB(a,r,g,b) \
;    ((D3DCOLOR)((((a)&0xff)<<24)|(((r)&0xff)<<16)|(((g)&0xff)<<8)|((b)&0xff)))
;D3DCOLOR_RGBA(r,g,b,a) D3DCOLOR_ARGB(a,r,g,b)
;D3DCOLOR_XRGB(r,g,b)   D3DCOLOR_ARGB(0xff,r,g,b)
;
;D3DCOLOR_XYUV(y,u,v)   D3DCOLOR_ARGB(0xff,y,u,v)
;D3DCOLOR_AYUV(a,y,u,v) D3DCOLOR_ARGB(a,y,u,v)

;// maps floating point channels (0.f to 1.f range) to D3DCOLOR
;D3DCOLOR_COLORVALUE(r,g,b,a) \
;    D3DCOLOR_RGBA((DWORD)((r)*255.f),(DWORD)((g)*255.f),(DWORD)((b)*255.f),(DWORD)((a)*255.f))

ifndef D3DVECTOR_DEFINED
D3DVECTOR struct
    x real4 ?
    y real4 ?
    z real4 ?
D3DVECTOR ENDS
D3DVECTOR_DEFINED equ <1>
endif

ifndef D3DCOLORVALUE_DEFINED
D3DCOLORVALUE struct
    r real4 ?
    g real4 ?
    b real4 ?
    a real4 ?
D3DCOLORVALUE ends
D3DCOLORVALUE_DEFINED equ <1>
endif

ifndef D3DRECT_DEFINED
D3DRECT struct
    x1 LONG ?
    y1 LONG ?
    x2 LONG ?
    y2 LONG ?
 D3DRECT ends
D3DRECT_DEFINED equ <1>
endif

;#ifndef D3DMATRIX_DEFINED
;typedef struct _D3DMATRIX {
;    union {
;        struct {
;            float        _11, _12, _13, _14;
;            float        _21, _22, _23, _24;
;            float        _31, _32, _33, _34;
;            float        _41, _42, _43, _44;

;        };
;        float m[4][4];
;    };
;} D3DMATRIX;
;D3DMATRIX_DEFINED
;#endif

D3DVIEWPORT9 struct
    X       DWORD       ?
    Y       DWORD       ?       ;/* Viewport Top left */
    dWidth  DWORD       ?
    dHeight DWORD       ?       ;/* Viewport Dimensions */
    MinZ    real4       ?       ;/* Min/max of clip Volume */
    MaxZ    real4       ?
D3DVIEWPORT9 ends

; * Values for clip fields.

;Max number of user clipping planes, supported in D3D.
D3DMAXUSERCLIPPLANES equ 32

; These bits could be ORed together to use with D3DRS_CLIPPLANEENABLE

D3DCLIPPLANE0 equ 1h
D3DCLIPPLANE1 equ 2h
D3DCLIPPLANE2 equ 4h
D3DCLIPPLANE3 equ 8h
D3DCLIPPLANE4 equ 10h
D3DCLIPPLANE5 equ 20h

; The following bits are used in the ClipUnion and ClipIntersection
; members of the D3DCLIPSTATUS9

D3DCS_LEFT     equ   00000001h
D3DCS_RIGHT    equ   00000002h
D3DCS_TOP      equ   00000004h
D3DCS_BOTTOM   equ   00000008h
D3DCS_FRONT    equ   00000010h
D3DCS_BACK     equ   00000020h
D3DCS_PLANE0   equ   00000040h
D3DCS_PLANE1   equ   00000080h
D3DCS_PLANE2   equ   00000100h
D3DCS_PLANE3   equ   00000200h
D3DCS_PLANE4   equ   00000400h
D3DCS_PLANE5   equ   00000800h

D3DCS_ALL equ 0fffh

D3DCLIPSTATUS9  struct
    ClipUnion        DWORD ?
    ClipIntersection DWORD ?
D3DCLIPSTATUS9 ends

D3DMATERIAL9 struct
       Diffuse  D3DCOLORVALUE  <>    ;Diffuse color RGBA 
       Ambient  D3DCOLORVALUE  <>    ;Ambient color RGB
       Specular D3DCOLORVALUE  <>    ;Specular 'shininess'
       Emissive D3DCOLORVALUE  <>    ;Emissive color RGB
       Power    real4     ?           ;Sharpness if specular highlight
D3DMATERIAL9 ends

;typedef enum _D3DLIGHTTYPE {
    D3DLIGHT_POINT          = 1
    D3DLIGHT_SPOT           = 2
    D3DLIGHT_DIRECTIONAL    = 3
;    D3DLIGHT_FORCE_DWORD    = 0x7fffffff, /* force 32-bit size enum */
;} D3DLIGHTTYPE;

D3DLIGHT9 struct 
    rType           DWORD           ?  ;D3DLIGHTTYPE Type of light source 
    Diffuse         D3DCOLORVALUE   <> ;Diffuse color of light 
    Specular        D3DCOLORVALUE   <> ;Specular color of light
    Ambient         D3DCOLORVALUE   <> ;Ambient color of light
    Position        D3DVECTOR       <> ;Position in world space
    Direction       D3DVECTOR       <> ;Direction in world space
    Range           REAL4           ?  ;Cutoff range
    Falloff         REAL4           ?  ;Falloff
    Attenuation0    REAL4           ?  ;Constant attenuation
    Attenuation1    REAL4           ?  ;Linear attenuation
    Attenuation2    REAL4           ?  ;Quadratic attenuation
    Theta           REAL4           ?  ;Inner angle of spotlight cone
    Phi             REAL4           ?  ;Outer angle of spotlight cone
D3DLIGHT9 ends

;Options for clearing

D3DCLEAR_TARGET      equ      00000001h  ;Clear target surface
D3DCLEAR_ZBUFFER     equ      00000002h  ;Clear target z buffer
D3DCLEAR_STENCIL     equ      00000004h  ;Clear stencil planes

;The following defines the rendering states

;typedef enum _D3DSHADEMODE {
    D3DSHADE_FLAT               equ 1
    D3DSHADE_GOURAUD            equ 2
    D3DSHADE_PHONG              equ 3
;    D3DSHADE_FORCE_DWORD        = 0x7fffffff, /* force 32-bit size enum */
;} D3DSHADEMODE;

;typedef enum _D3DFILLMODE {
    D3DFILL_POINT               equ 1
    D3DFILL_WIREFRAME           equ 2
    D3DFILL_SOLID               equ 3
;    D3DFILL_FORCE_DWORD         = 0x7fffffff, /* force 32-bit size enum */
;} D3DFILLMODE;

;typedef enum _D3DBLEND {
    D3DBLEND_ZERO               equ 1
    D3DBLEND_ONE                equ 2
    D3DBLEND_SRCCOLOR           equ 3
    D3DBLEND_INVSRCCOLOR        equ 4
    D3DBLEND_SRCALPHA           equ 5
    D3DBLEND_INVSRCALPHA        equ 6
    D3DBLEND_DESTALPHA          equ 7
    D3DBLEND_INVDESTALPHA       equ 8
    D3DBLEND_DESTCOLOR          equ 9
    D3DBLEND_INVDESTCOLOR       equ 10
    D3DBLEND_SRCALPHASAT        equ 11
    D3DBLEND_BOTHSRCALPHA       equ 12
    D3DBLEND_BOTHINVSRCALPHA    equ 13
    D3DBLEND_BLENDFACTOR        equ 14 ;Only supported if D3DPBLENDCAPS_BLENDFACTOR is on */
    D3DBLEND_INVBLENDFACTOR     equ 15 ;Only supported if D3DPBLENDCAPS_BLENDFACTOR is on *
;    D3DBLEND_FORCE_DWORD        = 0x7fffffff, /* force 32-bit size enum */
;} D3DBLEND;

;typedef enum _D3DBLENDOP {
    D3DBLENDOP_ADD              equ 1
    D3DBLENDOP_SUBTRACT         equ 2
    D3DBLENDOP_REVSUBTRACT      equ 3
    D3DBLENDOP_MIN              equ 4
    D3DBLENDOP_MAX              equ 5
;    D3DBLENDOP_FORCE_DWORD      = 0x7fffffff, /* force 32-bit size enum */
;} D3DBLENDOP;

;typedef enum _D3DTEXTUREADDRESS {
    D3DTADDRESS_WRAP            equ 1
    D3DTADDRESS_MIRROR          equ 2
    D3DTADDRESS_CLAMP           equ 3
    D3DTADDRESS_BORDER          equ 4
    D3DTADDRESS_MIRRORONCE      equ 5
;    D3DTADDRESS_FORCE_DWORD     = 0x7fffffff, /* force 32-bit size enum */
;} D3DTEXTUREADDRESS;

;typedef enum _D3DCULL {
    D3DCULL_NONE                equ 1
    D3DCULL_CW                  equ 2
    D3DCULL_CCW                 equ 3
;    D3DCULL_FORCE_DWORD         = 0x7fffffff, /* force 32-bit size enum */
;} D3DCULL;

;typedef enum _D3DCMPFUNC {
    D3DCMP_NEVER                equ 1
    D3DCMP_LESS                 equ 2
    D3DCMP_EQUAL                equ 3
    D3DCMP_LESSEQUAL            equ 4
    D3DCMP_GREATER              equ 5
    D3DCMP_NOTEQUAL             equ 6
    D3DCMP_GREATEREQUAL         equ 7
    D3DCMP_ALWAYS               equ 8
;    D3DCMP_FORCE_DWORD          = 0x7fffffff, /* force 32-bit size enum */
;} D3DCMPFUNC;

;typedef enum _D3DSTENCILOP {
    D3DSTENCILOP_KEEP           equ 1
    D3DSTENCILOP_ZERO           equ 2
    D3DSTENCILOP_REPLACE        equ 3
    D3DSTENCILOP_INCRSAT        equ 4
    D3DSTENCILOP_DECRSAT        equ 5
    D3DSTENCILOP_INVERT         equ 6
    D3DSTENCILOP_INCR           equ 7
    D3DSTENCILOP_DECR           equ 8
;    D3DSTENCILOP_FORCE_DWORD    = 0x7fffffff, /* force 32-bit size enum */
;} D3DSTENCILOP;

;typedef enum _D3DFOGMODE {
    D3DFOG_NONE                 equ 0
    D3DFOG_EXP                  equ 1
    D3DFOG_EXP2                 equ 2
    D3DFOG_LINEAR               equ 3
;    D3DFOG_FORCE_DWORD          = 0x7fffffff, /* force 32-bit size enum */
;} D3DFOGMODE;

;typedef enum _D3DZBUFFERTYPE {
    D3DZB_FALSE                 equ 0
    D3DZB_TRUE                  equ 1 ;Z buffering
    D3DZB_USEW                  equ 2 ;W buffering
;    D3DZB_FORCE_DWORD           = 0x7fffffff, /* force 32-bit size enum */
;} D3DZBUFFERTYPE;

;Primitives supported by draw-primitive API
;typedef enum _D3DPRIMITIVETYPE {
    D3DPT_POINTLIST             equ 1
    D3DPT_LINELIST              equ 2
    D3DPT_LINESTRIP             equ 3
    D3DPT_TRIANGLELIST          equ 4
    D3DPT_TRIANGLESTRIP         equ 5
    D3DPT_TRIANGLEFAN           equ 6
;    D3DPT_FORCE_DWORD           = 0x7fffffff, /* force 32-bit size enum */
;} D3DPRIMITIVETYPE;

;typedef enum _D3DTRANSFORMSTATETYPE {
    D3DTS_VIEW          equ 2
    D3DTS_PROJECTION    equ 3
    D3DTS_TEXTURE0      equ 16
    D3DTS_TEXTURE1      equ 17
    D3DTS_TEXTURE2      equ 18
    D3DTS_TEXTURE3      equ 19
    D3DTS_TEXTURE4      equ 20
    D3DTS_TEXTURE5      equ 21
    D3DTS_TEXTURE6      equ 22
    D3DTS_TEXTURE7      equ 23
;    D3DTS_FORCE_DWORD     = 0x7fffffff, /* force 32-bit size enum */
;} D3DTRANSFORMSTATETYPE;

;D3DTS_WORLDMATRIX(index) (D3DTRANSFORMSTATETYPE)(index + 256)
;D3DTS_WORLD  D3DTS_WORLDMATRIX(0)
;D3DTS_WORLD1 D3DTS_WORLDMATRIX(1)
;D3DTS_WORLD2 D3DTS_WORLDMATRIX(2)
;D3DTS_WORLD3 D3DTS_WORLDMATRIX(3)

;typedef enum _D3DRENDERSTATETYPE {
    D3DRS_ZENABLE                   equ 7    ;/* D3DZBUFFERTYPE (or TRUE/FALSE for legacy) */
    D3DRS_FILLMODE                  equ 8    ;/* D3DFILLMODE */
    D3DRS_SHADEMODE                 equ 9    ;/* D3DSHADEMODE */
    D3DRS_ZWRITEENABLE              equ 14   ;/* TRUE to enable z writes */
    D3DRS_ALPHATESTENABLE           equ 15   ;/* TRUE to enable alpha tests */
    D3DRS_LASTPIXEL                 equ 16   ;/* TRUE for last-pixel on lines */
    D3DRS_SRCBLEND                  equ 19   ;/* D3DBLEND */
    D3DRS_DESTBLEND                 equ 20   ;/* D3DBLEND */
    D3DRS_CULLMODE                  equ 22   ;/* D3DCULL */
    D3DRS_ZFUNC                     equ 23   ;/* D3DCMPFUNC */
    D3DRS_ALPHAREF                  equ 24   ;/* D3DFIXED */
    D3DRS_ALPHAFUNC                 equ 25   ;/* D3DCMPFUNC */
    D3DRS_DITHERENABLE              equ 26   ;/* TRUE to enable dithering */
    D3DRS_ALPHABLENDENABLE          equ 27   ;/* TRUE to enable alpha blending */
    D3DRS_FOGENABLE                 equ 28   ;/* TRUE to enable fog blending */
    D3DRS_SPECULARENABLE            equ 29   ;/* TRUE to enable specular */
    D3DRS_FOGCOLOR                  equ 34   ;/* D3DCOLOR */
    D3DRS_FOGTABLEMODE              equ 35   ;/* D3DFOGMODE */
    D3DRS_FOGSTART                  equ 36   ;/* Fog start (for both vertex and pixel fog) */
    D3DRS_FOGEND                    equ 37   ;/* Fog end      */
    D3DRS_FOGDENSITY                equ 38   ;/* Fog density  */
    D3DRS_RANGEFOGENABLE            equ 48   ;/* Enables range-based fog */
    D3DRS_STENCILENABLE             equ 52   ;/* BOOL enable/disable stenciling */
    D3DRS_STENCILFAIL               equ 53   ;/* D3DSTENCILOP to do if stencil test fails */
    D3DRS_STENCILZFAIL              equ 54   ;/* D3DSTENCILOP to do if stencil test passes and Z test fails */
    D3DRS_STENCILPASS               equ 55   ;/* D3DSTENCILOP to do if both stencil and Z tests pass */
    D3DRS_STENCILFUNC               equ 56   ;/* D3DCMPFUNC fn.  Stencil Test passes if ((ref & mask) stencilfn (stencil & mask)) is true */
    D3DRS_STENCILREF                equ 57   ;/* Reference value used in stencil test */
    D3DRS_STENCILMASK               equ 58   ;/* Mask value used in stencil test */
    D3DRS_STENCILWRITEMASK          equ 59   ;/* Write mask applied to values written to stencil buffer */
    D3DRS_TEXTUREFACTOR             equ 60   ;/* D3DCOLOR used for multi-texture blend */
    D3DRS_WRAP0                     equ 128  ;/* wrap for 1st texture coord. set */
    D3DRS_WRAP1                     equ 129  ;/* wrap for 2nd texture coord. set */
    D3DRS_WRAP2                     equ 130  ;/* wrap for 3rd texture coord. set */
    D3DRS_WRAP3                     equ 131  ;/* wrap for 4th texture coord. set */
    D3DRS_WRAP4                     equ 132  ;/* wrap for 5th texture coord. set */
    D3DRS_WRAP5                     equ 133  ;/* wrap for 6th texture coord. set */
    D3DRS_WRAP6                     equ 134  ;/* wrap for 7th texture coord. set */
    D3DRS_WRAP7                     equ 135  ;/* wrap for 8th texture coord. set */
    D3DRS_CLIPPING                  equ 136
    D3DRS_LIGHTING                  equ 137
    D3DRS_AMBIENT                   equ 139
    D3DRS_FOGVERTEXMODE             equ 140
    D3DRS_COLORVERTEX               equ 141
    D3DRS_LOCALVIEWER               equ 142
    D3DRS_NORMALIZENORMALS          equ 143
    D3DRS_DIFFUSEMATERIALSOURCE     equ 145
    D3DRS_SPECULARMATERIALSOURCE    equ 146
    D3DRS_AMBIENTMATERIALSOURCE     equ 147
    D3DRS_EMISSIVEMATERIALSOURCE    equ 148
    D3DRS_VERTEXBLEND               equ 151
    D3DRS_CLIPPLANEENABLE           equ 152
    D3DRS_POINTSIZE                 equ 154   ;/* float point size */
    D3DRS_POINTSIZE_MIN             equ 155   ;/* float point size min threshold */
    D3DRS_POINTSPRITEENABLE         equ 156   ;/* BOOL point texture coord control */
    D3DRS_POINTSCALEENABLE          equ 157   ;/* BOOL point size scale enable */
    D3DRS_POINTSCALE_A              equ 158   ;/* float point attenuation A value */
    D3DRS_POINTSCALE_B              equ 159   ;/* float point attenuation B value */
    D3DRS_POINTSCALE_C              equ 160   ;/* float point attenuation C value */
    D3DRS_MULTISAMPLEANTIALIAS      equ 161   ;// BOOL - set to do FSAA with multisample buffer
    D3DRS_MULTISAMPLEMASK           equ 162   ;// DWORD - per-sample enable/disable
    D3DRS_PATCHEDGESTYLE            equ 163   ;// Sets whether patch edges will use float style tessellation
    D3DRS_DEBUGMONITORTOKEN         equ 165   ;// DEBUG ONLY - token to debug monitor
    D3DRS_POINTSIZE_MAX             equ 166   ;/* float point size max threshold */
    D3DRS_INDEXEDVERTEXBLENDENABLE  equ 167
    D3DRS_COLORWRITEENABLE          equ 168   ;// per-channel write enable
    D3DRS_TWEENFACTOR               equ 170   ;// float tween factor
    D3DRS_BLENDOP                   equ 171   ;// D3DBLENDOP setting
    D3DRS_POSITIONDEGREE            equ 172   ;// NPatch position interpolation degree. D3DDEGREE_LINEAR or D3DDEGREE_CUBIC (default)
    D3DRS_NORMALDEGREE              equ 173   ;// NPatch normal interpolation degree. D3DDEGREE_LINEAR (default) or D3DDEGREE_QUADRATIC
    D3DRS_SCISSORTESTENABLE         equ 174
    D3DRS_SLOPESCALEDEPTHBIAS       equ 175
    D3DRS_ANTIALIASEDLINEENABLE     equ 176
    D3DRS_MINTESSELLATIONLEVEL      equ 178
    D3DRS_MAXTESSELLATIONLEVEL      equ 179
    D3DRS_ADAPTIVETESS_X            equ 180
    D3DRS_ADAPTIVETESS_Y            equ 181
    D3DRS_ADAPTIVETESS_Z            equ 182
    D3DRS_ADAPTIVETESS_W            equ 183
    D3DRS_ENABLEADAPTIVETESSELLATION equ 184
    D3DRS_TWOSIDEDSTENCILMODE       equ 185   ;/* BOOL enable/disable 2 sided stenciling */
    D3DRS_CCW_STENCILFAIL           equ 186   ;/* D3DSTENCILOP to do if ccw stencil test fails */
    D3DRS_CCW_STENCILZFAIL          equ 187   ;/* D3DSTENCILOP to do if ccw stencil test passes and Z test fails */
    D3DRS_CCW_STENCILPASS           equ 188   ;/* D3DSTENCILOP to do if both ccw stencil and Z tests pass */
    D3DRS_CCW_STENCILFUNC           equ 189   ;/* D3DCMPFUNC fn.  ccw Stencil Test passes if ((ref & mask) stencilfn (stencil & mask)) is true */
    D3DRS_COLORWRITEENABLE1         equ 190   ;/* Additional ColorWriteEnables for the devices that support D3DPMISCCAPS_INDEPENDENTWRITEMASKS */
    D3DRS_COLORWRITEENABLE2         equ 191   ;/* Additional ColorWriteEnables for the devices that support D3DPMISCCAPS_INDEPENDENTWRITEMASKS */
    D3DRS_COLORWRITEENABLE3         equ 192   ;/* Additional ColorWriteEnables for the devices that support D3DPMISCCAPS_INDEPENDENTWRITEMASKS */
    D3DRS_BLENDFACTOR               equ 193   ;/* D3DCOLOR used for a constant blend factor during alpha blending for devices that support D3DPBLENDCAPS_BLENDFACTOR */
    D3DRS_SRGBWRITEENABLE           equ 194   ;/* Enable rendertarget writes to be DE-linearized to SRGB (for formats that expose D3DUSAGE_QUERY_SRGBWRITE) */
    D3DRS_DEPTHBIAS                 equ 195
    D3DRS_WRAP8                     equ 198   ;/* Additional wrap states for vs_3_0+ attributes with D3DDECLUSAGE_TEXCOORD */
    D3DRS_WRAP9                     equ 199
    D3DRS_WRAP10                    equ 200
    D3DRS_WRAP11                    equ 201
    D3DRS_WRAP12                    equ 202
    D3DRS_WRAP13                    equ 203
    D3DRS_WRAP14                    equ 204
    D3DRS_WRAP15                    equ 205
    D3DRS_SEPARATEALPHABLENDENABLE  equ 206  ;/* TRUE to enable a separate blending function for the alpha channel */
    D3DRS_SRCBLENDALPHA             equ 207  ;/* SRC blend factor for the alpha channel when D3DRS_SEPARATEDESTALPHAENABLE is TRUE */
    D3DRS_DESTBLENDALPHA            equ 208  ;/* DST blend factor for the alpha channel when D3DRS_SEPARATEDESTALPHAENABLE is TRUE */
    D3DRS_BLENDOPALPHA              equ 209  ;/* Blending operation for the alpha channel when D3DRS_SEPARATEDESTALPHAENABLE is TRUE */


;    D3DRS_FORCE_DWORD               = 0x7fffffff, /* force 32-bit size enum */
;} D3DRENDERSTATETYPE;

;Maximum number of simultaneous render targets D3D supports
D3D_MAX_SIMULTANEOUS_RENDERTARGETS equ 4

;Values for material source
;typedef enum _D3DMATERIALCOLORSOURCE
;{
    D3DMCS_MATERIAL         equ 0            ;// Color from material is used
    D3DMCS_COLOR1           equ 1            ;// Diffuse vertex color is used
    D3DMCS_COLOR2           equ 2            ;// Specular vertex color is used
;    D3DMCS_FORCE_DWORD      = 0x7fffffff,   // force 32-bit size enum
;} D3DMATERIALCOLORSOURCE;

;Bias to apply to the texture coordinate set to apply a wrap to.
D3DRENDERSTATE_WRAPBIAS   equ            128

;Flags to construct the WRAP render states
D3DWRAP_U   equ 00000001h
D3DWRAP_V   equ 00000002h
D3DWRAP_W   equ 00000004h

;Flags to construct the WRAP render states for 1D thru 4D texture coordinates
D3DWRAPCOORD_0  equ 00000001h    ;same as D3DWRAP_U
D3DWRAPCOORD_1  equ 00000002h    ;same as D3DWRAP_V
D3DWRAPCOORD_2  equ 00000004h    ;same as D3DWRAP_W
D3DWRAPCOORD_3  equ 00000008h

;Flags to construct D3DRS_COLORWRITEENABLE
D3DCOLORWRITEENABLE_RED       equ 00000001h
D3DCOLORWRITEENABLE_GREEN     equ 00000002h
D3DCOLORWRITEENABLE_BLUE      equ 00000004h
D3DCOLORWRITEENABLE_ALPHA     equ 00000008h

;State enumerants for per-stage processing of fixed function pixel processing
;Two of these affect fixed function vertex processing as well: TEXTURETRANSFORMFLAGS and TEXCOORDINDEX.
;typedef enum _D3DTEXTURESTAGESTATETYPE
;{
    D3DTSS_COLOROP        equ  1 ;/* D3DTEXTUREOP - per-stage blending controls for color channels */
    D3DTSS_COLORARG1      equ  2 ;/* D3DTA_* (texture arg) */
    D3DTSS_COLORARG2      equ  3 ;/* D3DTA_* (texture arg) */
    D3DTSS_ALPHAOP        equ  4 ;/* D3DTEXTUREOP - per-stage blending controls for alpha channel */
    D3DTSS_ALPHAARG1      equ  5 ;/* D3DTA_* (texture arg) */
    D3DTSS_ALPHAARG2      equ  6 ;/* D3DTA_* (texture arg) */
    D3DTSS_BUMPENVMAT00   equ  7 ;/* float (bump mapping matrix) */
    D3DTSS_BUMPENVMAT01   equ  8 ;/* float (bump mapping matrix) */
    D3DTSS_BUMPENVMAT10   equ  9 ;/* float (bump mapping matrix) */
    D3DTSS_BUMPENVMAT11   equ 10 ;/* float (bump mapping matrix) */
    D3DTSS_TEXCOORDINDEX  equ 11 ;/* identifies which set of texture coordinates index this texture */
    D3DTSS_BUMPENVLSCALE  equ 22 ;/* float scale for bump map luminance */
    D3DTSS_BUMPENVLOFFSET equ 23 ;/* float offset for bump map luminance */
    D3DTSS_TEXTURETRANSFORMFLAGS equ 24 ;/* D3DTEXTURETRANSFORMFLAGS controls texture transform */
    D3DTSS_COLORARG0      equ 26 ;/* D3DTA_* third arg for triadic ops */
    D3DTSS_ALPHAARG0      equ 27 ;/* D3DTA_* third arg for triadic ops */
    D3DTSS_RESULTARG      equ 28 ;/* D3DTA_* arg for result (CURRENT or TEMP) */
    D3DTSS_CONSTANT       equ 32 ;/* Per-stage constant D3DTA_CONSTANT */


;    D3DTSS_FORCE_DWORD   = 0x7fffffff, /* force 32-bit size enum */
;} D3DTEXTURESTAGESTATETYPE;

;State enumerants for per-sampler texture processing.
;typedef enum _D3DSAMPLERSTATETYPE
;{
    D3DSAMP_ADDRESSU       equ 1  ;D3DTEXTUREADDRESS for U coordinate */
    D3DSAMP_ADDRESSV       equ 2  ;D3DTEXTUREADDRESS for V coordinate */
    D3DSAMP_ADDRESSW       equ 3  ;D3DTEXTUREADDRESS for W coordinate */
    D3DSAMP_BORDERCOLOR    equ 4  ;D3DCOLOR */
    D3DSAMP_MAGFILTER      equ 5  ;D3DTEXTUREFILTER filter to use for magnification */
    D3DSAMP_MINFILTER      equ 6  ;D3DTEXTUREFILTER filter to use for minification */
    D3DSAMP_MIPFILTER      equ 7  ;D3DTEXTUREFILTER filter to use between mipmaps during minification */
    D3DSAMP_MIPMAPLODBIAS  equ 8  ;float Mipmap LOD bias */
    D3DSAMP_MAXMIPLEVEL    equ 9  ;DWORD 0..(n-1) LOD index of largest map to use (0 == largest) */
    D3DSAMP_MAXANISOTROPY  equ 10 ;DWORD maximum anisotropy */
    D3DSAMP_SRGBTEXTURE    equ 11 ;Default = 0 (which means Gamma 1.0,
                                   ;no correction required.) else correct for
                                   ;Gamma = 2.2 */
    D3DSAMP_ELEMENTINDEX   equ 12 ;When multi-element texture is assigned to sampler, this
                                   ;indicates which element index to use.  Default = 0.  */
    D3DSAMP_DMAPOFFSET     equ 13 ;Offset in vertices in the pre-sampled displacement map.
                                   ;Only valid for D3DDMAPSAMPLER sampler  */
;    D3DSAMP_FORCE_DWORD   = 0x7fffffff, /* force 32-bit size enum */
;} D3DSAMPLERSTATETYPE;

;Special sampler which is used in the tesselator
D3DDMAPSAMPLER equ 256

;Samplers used in vertex shaders
D3DVERTEXTEXTURESAMPLER0 EQU D3DDMAPSAMPLER + 1
D3DVERTEXTEXTURESAMPLER1 EQU D3DDMAPSAMPLER + 2
D3DVERTEXTEXTURESAMPLER2 EQU D3DDMAPSAMPLER + 3
D3DVERTEXTEXTURESAMPLER3 EQU D3DDMAPSAMPLER + 4

;Values, used with D3DTSS_TEXCOORDINDEX, to specify that the vertex data(position
;and normal in the camera space) should be taken as texture coordinates
;Low 16 bits are used to specify texture coordinate index, to take the WRAP mode from

D3DTSS_TCI_PASSTHRU                     equ        00000000h
D3DTSS_TCI_CAMERASPACENORMAL            equ        00010000h
D3DTSS_TCI_CAMERASPACEPOSITION          equ        00020000h
D3DTSS_TCI_CAMERASPACEREFLECTIONVECTOR  equ        00030000h
D3DTSS_TCI_SPHEREMAP                    equ        00040000h

;Enumerations for COLOROP and ALPHAOP texture blending operations set in
;texture processing stage controls in D3DTSS.

;typedef enum _D3DTEXTUREOP
;{
    ;Control
    D3DTOP_DISABLE              equ 1      ;disables stage
    D3DTOP_SELECTARG1           equ 2      ;the default
    D3DTOP_SELECTARG2           equ 3

    ;Modulate
    D3DTOP_MODULATE             equ 4      ;multiply args together
    D3DTOP_MODULATE2X           equ 5      ;multiply and  1 bit
    D3DTOP_MODULATE4X           equ 6      ;multiply and  2 bits

    ;Add
    D3DTOP_ADD                  equ  7   ;add arguments together
    D3DTOP_ADDSIGNED            equ  8   ;add with -0.5 bias
    D3DTOP_ADDSIGNED2X          equ  9   ;as above but left  1 bit
    D3DTOP_SUBTRACT             equ 10   ;Arg1 - Arg2, with no saturation
    D3DTOP_ADDSMOOTH            equ 11   ;add 2 args, subtract product
                                        ;Arg1 + Arg2 - Arg1*Arg2
                                        ;= Arg1 + (1-Arg1)*Arg2

    ;Linear alpha blend: Arg1*(Alpha) + Arg2*(1-Alpha)
    D3DTOP_BLENDDIFFUSEALPHA    equ 12 ;iterated alpha
    D3DTOP_BLENDTEXTUREALPHA    equ 13 ;texture alpha
    D3DTOP_BLENDFACTORALPHA     equ 14 ;alpha from D3DRS_TEXTUREFACTOR

    ;Linear alpha blend with pre-multiplied arg1 input: Arg1 + Arg2*(1-Alpha)
    D3DTOP_BLENDTEXTUREALPHAPM  equ 15 ;texture alpha
    D3DTOP_BLENDCURRENTALPHA    equ 16 ;by alpha of current color

    ;Specular mapping
    D3DTOP_PREMODULATE            equ 17     ;modulate with next texture before use
    D3DTOP_MODULATEALPHA_ADDCOLOR equ 18     ;Arg1.RGB + Arg1.A*Arg2.RGB
                                            ;COLOROP only
    D3DTOP_MODULATECOLOR_ADDALPHA equ 19     ;Arg1.RGB*Arg2.RGB + Arg1.A
                                            ;COLOROP only
    D3DTOP_MODULATEINVALPHA_ADDCOLOR equ 20  ;(1-Arg1.A)*Arg2.RGB + Arg1.RGB
                                            ;COLOROP only
    D3DTOP_MODULATEINVCOLOR_ADDALPHA equ 21  ;(1-Arg1.RGB)*Arg2.RGB + Arg1.A
                                            ;COLOROP only

    ;Bump mapping
    D3DTOP_BUMPENVMAP           equ 22 ;per pixel env map perturbation
    D3DTOP_BUMPENVMAPLUMINANCE  equ 23 ;with luminance channel

    ;This can do either diffuse or specular bump mapping with correct input.
    ;Performs the function (Arg1.R*Arg2.R + Arg1.G*Arg2.G + Arg1.B*Arg2.B)
    ;where each component has been scaled and offset to make it signed.
    ;The result is replicated into all four (including alpha) channels.
    ;This is a valid COLOROP only.
    D3DTOP_DOTPRODUCT3          equ 24

    ;Triadic ops
    D3DTOP_MULTIPLYADD          equ 25 ;Arg0 + Arg1*Arg2
    D3DTOP_LERP                 equ 26 ;(Arg0)*Arg1 + (1-Arg0)*Arg2

;    D3DTOP_FORCE_DWORD = 0x7fffffff,
;} D3DTEXTUREOP;

;Values for COLORARG0,1,2, ALPHAARG0,1,2, and RESULTARG texture blending
;operations set in texture processing stage controls in D3DRENDERSTATE.

D3DTA_SELECTMASK     equ    0000000fh  ;mask for arg selector
D3DTA_DIFFUSE        equ    00000000h  ;select diffuse color (read only)
D3DTA_CURRENT        equ    00000001h  ;select stage destination register (read/write)
D3DTA_TEXTURE        equ    00000002h  ;select texture color (read only)
D3DTA_TFACTOR        equ    00000003h  ;select D3DRS_TEXTUREFACTOR (read only)
D3DTA_SPECULAR       equ    00000004h  ;select specular color (read only)
D3DTA_TEMP           equ    00000005h  ;select temporary register color (read/write)
D3DTA_CONSTANT       equ    00000006h  ;select texture stage constant
D3DTA_COMPLEMENT     equ    00000010h  ;take 1.0 - x (read modifier)
D3DTA_ALPHAREPLICATE equ    00000020h  ;replicate alpha to color components (read modifier)

;Values for D3DSAMP_***FILTER texture stage states
;typedef enum _D3DTEXTUREFILTERTYPE
;{
    D3DTEXF_NONE            equ 0    ;filtering disabled (valid for mip filter only)
    D3DTEXF_POINT           equ 1    ;nearest
    D3DTEXF_LINEAR          equ 2    ;linear interpolation
    D3DTEXF_ANISOTROPIC     equ 3    ;anisotropic
    D3DTEXF_PYRAMIDALQUAD   equ 6    ;4-sample tent
    D3DTEXF_GAUSSIANQUAD    equ 7    ;4-sample gaussian
;    D3DTEXF_FORCE_DWORD     = 0x7fffffff,   // force 32-bit size enum
;} D3DTEXTUREFILTERTYPE;

;Bits for Flags in ProcessVertices call

D3DPV_DONOTCOPYDATA  equ 1

;-------------------------------------------------------------------

;Flexible vertex format bits

D3DFVF_RESERVED0         equ 0x001h
D3DFVF_POSITION_MASK     equ 0x400Eh
D3DFVF_XYZ               equ 0x002h
D3DFVF_XYZRHW            equ 0x004h
D3DFVF_XYZB1             equ 0x006h
D3DFVF_XYZB2             equ 0x008h
D3DFVF_XYZB3             equ 0x00ah
D3DFVF_XYZB4             equ 0x00ch
D3DFVF_XYZB5             equ 0x00eh
D3DFVF_XYZW              equ 0x4002h
D3DFVF_NORMAL            equ 0x010h
D3DFVF_PSIZE             equ 0x020h
D3DFVF_DIFFUSE           equ 0x040h
D3DFVF_SPECULAR          equ 0x080h
D3DFVF_TEXCOUNT_MASK     equ 0xf00h
D3DFVF_TEXCOUNT_SHIFT    equ 8h
D3DFVF_TEX0              equ 0x000h
D3DFVF_TEX1              equ 0x100h
D3DFVF_TEX2              equ 0x200h
D3DFVF_TEX3              equ 0x300h
D3DFVF_TEX4              equ 0x400h
D3DFVF_TEX5              equ 0x500h
D3DFVF_TEX6              equ 0x600h
D3DFVF_TEX7              equ 0x700h
D3DFVF_TEX8              equ 0x800h
D3DFVF_LASTBETA_UBYTE4   equ 0x1000h
D3DFVF_LASTBETA_D3DCOLOR equ 0x8000h
D3DFVF_RESERVED2         equ 0x6000h  // 2 reserved bits

;---------------------------------------------------------------------
;Vertex Shaders

;Vertex shader declaration

;Vertex element semantics

;typedef enum _D3DDECLUSAGE
;{
;    D3DDECLUSAGE_POSITION = 0,
;    D3DDECLUSAGE_BLENDWEIGHT,   ;1
;    D3DDECLUSAGE_BLENDINDICES,  ;2
;    D3DDECLUSAGE_NORMAL,        ;3
;    D3DDECLUSAGE_PSIZE,         ;4
;    D3DDECLUSAGE_TEXCOORD,      ;5
;    D3DDECLUSAGE_TANGENT,       ;6
;    D3DDECLUSAGE_BINORMAL,      ;7
;    D3DDECLUSAGE_TESSFACTOR,    ;8
;    D3DDECLUSAGE_POSITIONT,     ;9
;    D3DDECLUSAGE_COLOR,         ;10
;    D3DDECLUSAGE_FOG,           ;11
;    D3DDECLUSAGE_DEPTH,         ;12
;    D3DDECLUSAGE_SAMPLE,        ;13
;} D3DDECLUSAGE;

;MAXD3DDECLUSAGE         D3DDECLUSAGE_SAMPLE
;MAXD3DDECLUSAGEINDEX    15
;MAXD3DDECLLENGTH        64 // does not include "end" marker vertex element

;typedef enum _D3DDECLMETHOD
;{
;    D3DDECLMETHOD_DEFAULT = 0,
;    D3DDECLMETHOD_PARTIALU,
;    D3DDECLMETHOD_PARTIALV,
;    D3DDECLMETHOD_CROSSUV,    // Normal
;    D3DDECLMETHOD_UV,
;    D3DDECLMETHOD_LOOKUP,               // Lookup a displacement map
;    D3DDECLMETHOD_LOOKUPPRESAMPLED,     // Lookup a pre-sampled displacement map
;} D3DDECLMETHOD;

;MAXD3DDECLMETHOD D3DDECLMETHOD_LOOKUPPRESAMPLED

;// Declarations for _Type fields
;//
;typedef enum _D3DDECLTYPE
;{
    D3DDECLTYPE_FLOAT1    equ  0  ;1D float expanded to (value, 0., 0., 1.)
    D3DDECLTYPE_FLOAT2    equ  1  ;2D float expanded to (value, value, 0., 1.)
    D3DDECLTYPE_FLOAT3    equ  2  ;3D float expanded to (value, value, value, 1.)
    D3DDECLTYPE_FLOAT4    equ  3  ;4D float
    D3DDECLTYPE_D3DCOLOR  equ  4  ;4D packed unsigned bytes mapped to 0. to 1. range
                                  ;Input is in D3DCOLOR format (ARGB) expanded to (R, G, B, A)
    D3DDECLTYPE_UBYTE4    equ  5  ;4D unsigned byte
    D3DDECLTYPE_SHORT2    equ  6  ;2D signed short expanded to (value, value, 0., 1.)
    D3DDECLTYPE_SHORT4    equ  7  ;4D signed short

;The following types are valid only with vertex shaders >= 2.0


    D3DDECLTYPE_UBYTE4N   equ  8  ;Each of 4 bytes is normalized by dividing to 255.0
    D3DDECLTYPE_SHORT2N   equ  9  ;2D signed short normalized (v[0]/32767.0,v[1]/32767.0,0,1)
    D3DDECLTYPE_SHORT4N   equ 10  ;4D signed short normalized (v[0]/32767.0,v[1]/32767.0,v[2]/32767.0,v[3]/32767.0)
    D3DDECLTYPE_USHORT2N  equ 11  ;2D unsigned short normalized (v[0]/65535.0,v[1]/65535.0,0,1)
    D3DDECLTYPE_USHORT4N  equ 12  ;4D unsigned short normalized (v[0]/65535.0,v[1]/65535.0,v[2]/65535.0,v[3]/65535.0)
    D3DDECLTYPE_UDEC3     equ 13  ;3D unsigned 10 10 10 format expanded to (value, value, value, 1)
    D3DDECLTYPE_DEC3N     equ 14  ;3D signed 10 10 10 format normalized and expanded to (v[0]/511.0, v[1]/511.0, v[2]/511.0, 1)
    D3DDECLTYPE_FLOAT16_2 equ 15  ;Two 16-bit floating point values, expanded to (value, value, 0, 1)
    D3DDECLTYPE_FLOAT16_4 equ 16  ;Four 16-bit floating point values
    D3DDECLTYPE_UNUSED    equ 17  ;When the type field in a decl is unused.
;} D3DDECLTYPE;

MAXD3DDECLTYPE   equ   D3DDECLTYPE_UNUSED

D3DVERTEXELEMENT9 struct
    Stream      WORD    ?     ;Stream index
    jOffset     WORD    ?     ;Offset in the stream in bytes
    jType       BYTE    ?     ;Data type
    Method      BYTE    ?     ;Processing method
    Usage       BYTE    ?     ;Semantics
    UsageIndex  BYTE    ?     ;Semantic index
D3DVERTEXELEMENT9 ends
;, *LPD3DVERTEXELEMENT9;

;This is used to initialize the last vertex element in a vertex declaration
;array

;D3DDECL_END() {0xFF,0,D3DDECLTYPE_UNUSED,0,0,0}

;Maximum supported number of texture coordinate sets
D3DDP_MAXTEXCOORD   equ 8

;---------------------------------------------------------------------
;Values for IDirect3DDevice9::SetStreamSourceFreq's Setting parameter
;---------------------------------------------------------------------
D3DSTREAMSOURCE_INDEXEDDATA  equ 20000000h
D3DSTREAMSOURCE_INSTANCEDATA equ 40000000h


;---------------------------------------------------------------------
;
; The internal format of Pixel Shader (PS) & Vertex Shader (VS)
; Instruction Tokens is defined in the Direct3D Device Driver Kit
;
;---------------------------------------------------------------------


; Instruction Token Bit Definitions

D3DSI_OPCODE_MASK      equ 0000FFFFh

D3DSI_INSTLENGTH_MASK   equ 0F000000h
D3DSI_INSTLENGTH_SHIFT  equ 24

;typedef enum _D3DSHADER_INSTRUCTION_OPCODE_TYPE
;{
;    D3DSIO_NOP          = 0,
;    D3DSIO_MOV          ,
;    D3DSIO_ADD          ,
;    D3DSIO_SUB          ,
;    D3DSIO_MAD          ,
;    D3DSIO_MUL          ,
;    D3DSIO_RCP          ,
;    D3DSIO_RSQ          ,
;    D3DSIO_DP3          ,
;    D3DSIO_DP4          ,
;    D3DSIO_MIN          ,
;    D3DSIO_MAX          ,
;    D3DSIO_SLT          ,
;    D3DSIO_SGE          ,
;    D3DSIO_EXP          ,
;    D3DSIO_LOG          ,
;    D3DSIO_LIT          ,
;    D3DSIO_DST          ,
;    D3DSIO_LRP          ,
;    D3DSIO_FRC          ,
;    D3DSIO_M4x4         ,
;    D3DSIO_M4x3         ,
;    D3DSIO_M3x4         ,
;    D3DSIO_M3x3         ,
;    D3DSIO_M3x2         ,
;    D3DSIO_CALL         ,
;    D3DSIO_CALLNZ       ,
;    D3DSIO_LOOP         ,
;    D3DSIO_RET          ,
;    D3DSIO_ENDLOOP      ,
;    D3DSIO_LABEL        ,
;    D3DSIO_DCL          ,
;    D3DSIO_POW          ,
;    D3DSIO_CRS          ,
;    D3DSIO_SGN          ,
;    D3DSIO_ABS          ,
;    D3DSIO_NRM          ,
;    D3DSIO_SINCOS       ,
;    D3DSIO_REP          ,
;    D3DSIO_ENDREP       ,
;    D3DSIO_IF           ,
;    D3DSIO_IFC          ,
;    D3DSIO_ELSE         ,
;    D3DSIO_ENDIF        ,
;    D3DSIO_BREAK        ,
;    D3DSIO_BREAKC       ,
;    D3DSIO_MOVA         ,
;    D3DSIO_DEFB         ,
;    D3DSIO_DEFI         ,
;
;    D3DSIO_TEXCOORD     = 64,
;    D3DSIO_TEXKILL      ,
;    D3DSIO_TEX          ,
;    D3DSIO_TEXBEM       ,
;    D3DSIO_TEXBEML      ,
;    D3DSIO_TEXREG2AR    ,
;    D3DSIO_TEXREG2GB    ,
;    D3DSIO_TEXM3x2PAD   ,
;    D3DSIO_TEXM3x2TEX   ,
;    D3DSIO_TEXM3x3PAD   ,
;    D3DSIO_TEXM3x3TEX   ,
;    D3DSIO_RESERVED0    ,
;    D3DSIO_TEXM3x3SPEC  ,
;    D3DSIO_TEXM3x3VSPEC ,
;    D3DSIO_EXPP         ,
;    D3DSIO_LOGP         ,
;    D3DSIO_CND          ,
;    D3DSIO_DEF          ,
;    D3DSIO_TEXREG2RGB   ,
;    D3DSIO_TEXDP3TEX    ,
;    D3DSIO_TEXM3x2DEPTH ,
;    D3DSIO_TEXDP3       ,
;    D3DSIO_TEXM3x3      ,
;    D3DSIO_TEXDEPTH     ,
;    D3DSIO_CMP          ,
;    D3DSIO_BEM          ,
;    D3DSIO_DP2ADD       ,
;    D3DSIO_DSX          ,
;    D3DSIO_DSY          ,
;    D3DSIO_TEXLDD       ,
;    D3DSIO_SETP         ,
;    D3DSIO_TEXLDL       ,
;    D3DSIO_BREAKP       ,
;
;    D3DSIO_PHASE        = 0xFFFD,
;    D3DSIO_COMMENT      = 0xFFFE,
;    D3DSIO_END          = 0xFFFF,

;    D3DSIO_FORCE_DWORD  = 0x7fffffff,   // force 32-bit size enum
;} D3DSHADER_INSTRUCTION_OPCODE_TYPE;

;//---------------------------------------------------------------------
;// Use these constants with D3DSIO_SINCOS macro as SRC2, SRC3
;//
;D3DSINCOSCONST1 -1.5500992e-006f, -2.1701389e-005f,  0.0026041667f, 0.00026041668f
;D3DSINCOSCONST2 -0.020833334f, -0.12500000f, 1.0f, 0.50000000f

;//---------------------------------------------------------------------
;// Co-Issue Instruction Modifier - if set then this instruction is to be
;// issued in parallel with the previous instruction(s) for which this bit
;// is not set.
;//
D3DSI_COISSUE    equ       40000000h

;//---------------------------------------------------------------------
;// Opcode specific controls

D3DSP_OPCODESPECIFICCONTROL_MASK  equ 00ff0000h
D3DSP_OPCODESPECIFICCONTROL_SHIFT equ 16

;// ps_2_0 texld controls
;D3DSI_TEXLD_PROJECT (0x01 << D3DSP_OPCODESPECIFICCONTROL_SHIFT)
;D3DSI_TEXLD_BIAS    (0x02 << D3DSP_OPCODESPECIFICCONTROL_SHIFT)

;// Comparison for dynamic conditional instruction opcodes (i.e. if, breakc)
;typedef enum _D3DSHADER_COMPARISON
;{
;                         // < = >
;    D3DSPC_RESERVED0= 0, // 0 0 0
;    D3DSPC_GT       = 1, // 0 0 1
;    D3DSPC_EQ       = 2, // 0 1 0
;    D3DSPC_GE       = 3, // 0 1 1
;    D3DSPC_LT       = 4, // 1 0 0
;    D3DSPC_NE       = 5, // 1 0 1
;    D3DSPC_LE       = 6, // 1 1 0
;    D3DSPC_RESERVED1= 7  // 1 1 1
;} D3DSHADER_COMPARISON;

;// Comparison is part of instruction opcode token:
;D3DSHADER_COMPARISON_SHIFT D3DSP_OPCODESPECIFICCONTROL_SHIFT
;D3DSHADER_COMPARISON_MASK  (0x7<<D3DSHADER_COMPARISON_SHIFT)
;
;//---------------------------------------------------------------------
;// Predication flags on instruction token
;D3DSHADER_INSTRUCTION_PREDICATED    (0x1 << 28)
;
;//---------------------------------------------------------------------
;// DCL Info Token Controls
;
;// For dcl info tokens requiring a semantic (usage + index)
;D3DSP_DCL_USAGE_SHIFT 0
;D3DSP_DCL_USAGE_MASK  0x0000000f
;
;D3DSP_DCL_USAGEINDEX_SHIFT 16
;D3DSP_DCL_USAGEINDEX_MASK  0x000f0000
;
;// DCL pixel shader sampler info token.
;D3DSP_TEXTURETYPE_SHIFT 27
;D3DSP_TEXTURETYPE_MASK  0x78000000
;
;typedef enum _D3DSAMPLER_TEXTURE_TYPE
;{
;    D3DSTT_UNKNOWN = 0<<D3DSP_TEXTURETYPE_SHIFT, // uninitialized value
;    D3DSTT_2D      = 2<<D3DSP_TEXTURETYPE_SHIFT, // dcl_2d s# (for declaring a 2-D texture)
;    D3DSTT_CUBE    = 3<<D3DSP_TEXTURETYPE_SHIFT, // dcl_cube s# (for declaring a cube texture)
;    D3DSTT_VOLUME  = 4<<D3DSP_TEXTURETYPE_SHIFT, // dcl_volume s# (for declaring a volume texture)
;    D3DSTT_FORCE_DWORD  = 0x7fffffff,      // force 32-bit size enum
;} D3DSAMPLER_TEXTURE_TYPE
;
;//---------------------------------------------------------------------
;// Parameter Token Bit Definitions
;//
D3DSP_REGNUM_MASK  equ     000007FFh

;// destination parameter write mask
D3DSP_WRITEMASK_0       equ 00010000h  ; Component 0 (X;Red)
D3DSP_WRITEMASK_1       equ 00020000h  ; Component 1 (Y;Green)
D3DSP_WRITEMASK_2       equ 00040000h  ; Component 2 (Z;Blue)
D3DSP_WRITEMASK_3       equ 00080000h  ; Component 3 (W;Alpha)
D3DSP_WRITEMASK_ALL     equ 000F0000h  ; All Components

; destination parameter modifiers
D3DSP_DSTMOD_SHIFT      equ 20
D3DSP_DSTMOD_MASK       equ 00F00000h

;// Bit masks for destination parameter modifiers
;   D3DSPDM_NONE                 (0<<D3DSP_DSTMOD_SHIFT) // nop
;   D3DSPDM_SATURATE             (1<<D3DSP_DSTMOD_SHIFT) // clamp to 0. to 1. range
;   D3DSPDM_PARTIALPRECISION     (2<<D3DSP_DSTMOD_SHIFT) // Partial precision hint
;   D3DSPDM_MSAMPCENTROID        (4<<D3DSP_DSTMOD_SHIFT) // Relevant to multisampling only:
;                                                                //      When the pixel center is not covered, sample
;                                                                //      attribute or compute gradients/LOD
;                                                                //      using multisample "centroid" location.
;                                                                //      "Centroid" is some location within the covered
;                                                                //      region of the pixel.

;// destination parameter 
;D3DSP_DSTSHIFT_SHIFT    24
;D3DSP_DSTSHIFT_MASK     0x0F000000;
;
;// destination/source parameter register type
;D3DSP_REGTYPE_SHIFT     28
;D3DSP_REGTYPE_SHIFT2    8
;D3DSP_REGTYPE_MASK      0x70000000
;D3DSP_REGTYPE_MASK2     0x00001800

;typedef enum _D3DSHADER_PARAM_REGISTER_TYPE
;{
;    D3DSPR_TEMP           =  0, // Temporary Register File
;    D3DSPR_INPUT          =  1, // Input Register File
;    D3DSPR_CONST          =  2, // Constant Register File
;    D3DSPR_ADDR           =  3, // Address Register (VS)
;    D3DSPR_TEXTURE        =  3, // Texture Register File (PS)
;    D3DSPR_RASTOUT        =  4, // Rasterizer Register File
;    D3DSPR_ATTROUT        =  5, // Attribute Output Register File
;    D3DSPR_TEXCRDOUT      =  6, // Texture Coordinate Output Register File
;    D3DSPR_OUTPUT         =  6, // Output register file for VS3.0+
;    D3DSPR_CONSTINT       =  7, // Constant Integer Vector Register File
;    D3DSPR_COLOROUT       =  8, // Color Output Register File
;    D3DSPR_DEPTHOUT       =  9, // Depth Output Register File
;    D3DSPR_SAMPLER        = 10, // Sampler State Register File
;    D3DSPR_CONST2         = 11, // Constant Register File  2048 - 4095
;    D3DSPR_CONST3         = 12, // Constant Register File  4096 - 6143
;    D3DSPR_CONST4         = 13, // Constant Register File  6144 - 8191
;    D3DSPR_CONSTBOOL      = 14, // Constant Boolean register file
;    D3DSPR_LOOP           = 15, // Loop counter register file
;    D3DSPR_TEMPFLOAT16    = 16, // 16-bit float temp register file
;    D3DSPR_MISCTYPE       = 17, // Miscellaneous (single) registers.
;    D3DSPR_LABEL          = 18, // Label
;    D3DSPR_PREDICATE      = 19, // Predicate register
;    D3DSPR_FORCE_DWORD  = 0x7fffffff,         // force 32-bit size enum
;} D3DSHADER_PARAM_REGISTER_TYPE;

;// The miscellaneous register file (D3DSPR_MISCTYPES)
;// contains register types for which there is only ever one
;// register (i.e. the register # is not needed).
;// Rather than use up additional register types for such
;// registers, they are defined
;// as particular offsets into the misc. register file:
;typedef enum _D3DSHADER_MISCTYPE_OFFSETS
;{
;    D3DSMO_POSITION   = 0, // Input position x,y,z,rhw (PS)
;    D3DSMO_FACE   = 1, // Floating point primitive area (PS)
;} D3DSHADER_MISCTYPE_OFFSETS;

;// Register offsets in the Rasterizer Register File
;//
;typedef enum _D3DVS_RASTOUT_OFFSETS
;{
;    D3DSRO_POSITION = 0,
;    D3DSRO_FOG,
;    D3DSRO_POINT_SIZE,
;    D3DSRO_FORCE_DWORD  = 0x7fffffff,         // force 32-bit size enum
;} D3DVS_RASTOUT_OFFSETS;

;// Source operand addressing modes
;
;D3DVS_ADDRESSMODE_SHIFT 13
;D3DVS_ADDRESSMODE_MASK  (1 << D3DVS_ADDRESSMODE_SHIFT)
;
;typedef enum _D3DVS_ADDRESSMODE_TYPE
;{
;    D3DVS_ADDRMODE_ABSOLUTE  = (0 << D3DVS_ADDRESSMODE_SHIFT),
;    D3DVS_ADDRMODE_RELATIVE  = (1 << D3DVS_ADDRESSMODE_SHIFT),
;    D3DVS_ADDRMODE_FORCE_DWORD = 0x7fffffff, // force 32-bit size enum
;} D3DVS_ADDRESSMODE_TYPE;

;D3DSHADER_ADDRESSMODE_SHIFT 13
;D3DSHADER_ADDRESSMODE_MASK  (1 << D3DSHADER_ADDRESSMODE_SHIFT)

;typedef enum _D3DSHADER_ADDRESSMODE_TYPE
;{
;    D3DSHADER_ADDRMODE_ABSOLUTE  = (0 << D3DSHADER_ADDRESSMODE_SHIFT),
;    D3DSHADER_ADDRMODE_RELATIVE  = (1 << D3DSHADER_ADDRESSMODE_SHIFT),
;    D3DSHADER_ADDRMODE_FORCE_DWORD = 0x7fffffff, // force 32-bit size enum
;} D3DSHADER_ADDRESSMODE_TYPE;

;// Source operand swizzle definitions
;//
;D3DVS_SWIZZLE_SHIFT     16
;D3DVS_SWIZZLE_MASK      0x00FF0000

;// The following bits define where to take component X from:

;D3DVS_X_X       (0 << D3DVS_SWIZZLE_SHIFT)
;D3DVS_X_Y       (1 << D3DVS_SWIZZLE_SHIFT)
;D3DVS_X_Z       (2 << D3DVS_SWIZZLE_SHIFT)
;D3DVS_X_W       (3 << D3DVS_SWIZZLE_SHIFT)

;// The following bits define where to take component Y from:

;D3DVS_Y_X       (0 << (D3DVS_SWIZZLE_SHIFT + 2))
;D3DVS_Y_Y       (1 << (D3DVS_SWIZZLE_SHIFT + 2))
;D3DVS_Y_Z       (2 << (D3DVS_SWIZZLE_SHIFT + 2))
;D3DVS_Y_W       (3 << (D3DVS_SWIZZLE_SHIFT + 2))

;// The following bits define where to take component Z from:

;D3DVS_Z_X       (0 << (D3DVS_SWIZZLE_SHIFT + 4))
;D3DVS_Z_Y       (1 << (D3DVS_SWIZZLE_SHIFT + 4))
;D3DVS_Z_Z       (2 << (D3DVS_SWIZZLE_SHIFT + 4))
;D3DVS_Z_W       (3 << (D3DVS_SWIZZLE_SHIFT + 4))

;// The following bits define where to take component W from:

;D3DVS_W_X       (0 << (D3DVS_SWIZZLE_SHIFT + 6))
;D3DVS_W_Y       (1 << (D3DVS_SWIZZLE_SHIFT + 6))
;D3DVS_W_Z       (2 << (D3DVS_SWIZZLE_SHIFT + 6))
;D3DVS_W_W       (3 << (D3DVS_SWIZZLE_SHIFT + 6))

;// Value when there is no swizzle (X is taken from X, Y is taken from Y,
;// Z is taken from Z, W is taken from W
;//
;D3DVS_NOSWIZZLE (D3DVS_X_X | D3DVS_Y_Y | D3DVS_Z_Z | D3DVS_W_W)

;// source parameter swizzle
;D3DSP_SWIZZLE_SHIFT     16
;D3DSP_SWIZZLE_MASK      0x00FF0000
;
;D3DSP_NOSWIZZLE \
;    ( (0 << (D3DSP_SWIZZLE_SHIFT + 0)) | \
;      (1 << (D3DSP_SWIZZLE_SHIFT + 2)) | \
;      (2 << (D3DSP_SWIZZLE_SHIFT + 4)) | \
;      (3 << (D3DSP_SWIZZLE_SHIFT + 6)) )
;
;// pixel-shader swizzle ops
;D3DSP_REPLICATERED \
;    ( (0 << (D3DSP_SWIZZLE_SHIFT + 0)) | \
;      (0 << (D3DSP_SWIZZLE_SHIFT + 2)) | \
;      (0 << (D3DSP_SWIZZLE_SHIFT + 4)) | \
;      (0 << (D3DSP_SWIZZLE_SHIFT + 6)) )

;D3DSP_REPLICATEGREEN \
;    ( (1 << (D3DSP_SWIZZLE_SHIFT + 0)) | \
;      (1 << (D3DSP_SWIZZLE_SHIFT + 2)) | \
;      (1 << (D3DSP_SWIZZLE_SHIFT + 4)) | \
;      (1 << (D3DSP_SWIZZLE_SHIFT + 6)) )

;D3DSP_REPLICATEBLUE \
;    ( (2 << (D3DSP_SWIZZLE_SHIFT + 0)) | \
;      (2 << (D3DSP_SWIZZLE_SHIFT + 2)) | \
;      (2 << (D3DSP_SWIZZLE_SHIFT + 4)) | \
;      (2 << (D3DSP_SWIZZLE_SHIFT + 6)) )

;D3DSP_REPLICATEALPHA \
;    ( (3 << (D3DSP_SWIZZLE_SHIFT + 0)) | \
;      (3 << (D3DSP_SWIZZLE_SHIFT + 2)) | \
;      (3 << (D3DSP_SWIZZLE_SHIFT + 4)) | \
;      (3 << (D3DSP_SWIZZLE_SHIFT + 6)) )

;// source parameter modifiers
;D3DSP_SRCMOD_SHIFT      24
;D3DSP_SRCMOD_MASK       0x0F000000

;typedef enum _D3DSHADER_PARAM_SRCMOD_TYPE
;{
;    D3DSPSM_NONE    = 0<<D3DSP_SRCMOD_SHIFT, // nop
;    D3DSPSM_NEG     = 1<<D3DSP_SRCMOD_SHIFT, // negate
;    D3DSPSM_BIAS    = 2<<D3DSP_SRCMOD_SHIFT, // bias
;    D3DSPSM_BIASNEG = 3<<D3DSP_SRCMOD_SHIFT, // bias and negate
;    D3DSPSM_SIGN    = 4<<D3DSP_SRCMOD_SHIFT, // sign
;    D3DSPSM_SIGNNEG = 5<<D3DSP_SRCMOD_SHIFT, // sign and negate
;    D3DSPSM_COMP    = 6<<D3DSP_SRCMOD_SHIFT, // complement
;    D3DSPSM_X2      = 7<<D3DSP_SRCMOD_SHIFT, // *2
;    D3DSPSM_X2NEG   = 8<<D3DSP_SRCMOD_SHIFT, // *2 and negate
;    D3DSPSM_DZ      = 9<<D3DSP_SRCMOD_SHIFT, // divide through by z component
;    D3DSPSM_DW      = 10<<D3DSP_SRCMOD_SHIFT, // divide through by w component
;    D3DSPSM_ABS     = 11<<D3DSP_SRCMOD_SHIFT, // abs()
;    D3DSPSM_ABSNEG  = 12<<D3DSP_SRCMOD_SHIFT, // -abs()
;    D3DSPSM_NOT     = 13<<D3DSP_SRCMOD_SHIFT, // for predicate register: "!p0"
;    D3DSPSM_FORCE_DWORD = 0x7fffffff,        // force 32-bit size enum
;} D3DSHADER_PARAM_SRCMOD_TYPE;

;// pixel shader version token
;D3DPS_VERSION(_Major,_Minor) (0xFFFF0000|((_Major)<<8)|(_Minor))

;// vertex shader version token
;D3DVS_VERSION(_Major,_Minor) (0xFFFE0000|((_Major)<<8)|(_Minor))

;// extract major/minor from version cap
;D3DSHADER_VERSION_MAJOR(_Version) (((_Version)>>8)&0xFF)
;D3DSHADER_VERSION_MINOR(_Version) (((_Version)>>0)&0xFF)

;// destination/source parameter register type
;D3DSI_COMMENTSIZE_SHIFT     16
;D3DSI_COMMENTSIZE_MASK      0x7FFF0000
;D3DSHADER_COMMENT(_DWordSize) \
;    ((((_DWordSize)<<D3DSI_COMMENTSIZE_SHIFT)&D3DSI_COMMENTSIZE_MASK)|D3DSIO_COMMENT)

;// pixel/vertex shader end token
;D3DPS_END()  0x0000FFFF
;D3DVS_END()  0x0000FFFF


;//---------------------------------------------------------------------

;// High order surfaces
;//
;typedef enum _D3DBASISTYPE
;{
;   D3DBASIS_BEZIER      = 0,
;   D3DBASIS_BSPLINE     = 1,
;   D3DBASIS_CATMULL_ROM = 2, /* In D3D8 this used to be D3DBASIS_INTERPOLATE */
;   D3DBASIS_FORCE_DWORD = 0x7fffffff,
;} D3DBASISTYPE;

;typedef enum _D3DDEGREETYPE
;{
;   D3DDEGREE_LINEAR      = 1,
;   D3DDEGREE_QUADRATIC   = 2,
;   D3DDEGREE_CUBIC       = 3,
;   D3DDEGREE_QUINTIC     = 5,
;   D3DDEGREE_FORCE_DWORD = 0x7fffffff,
;} D3DDEGREETYPE;

;typedef enum _D3DPATCHEDGESTYLE
;{
;   D3DPATCHEDGE_DISCRETE    = 0,
;   D3DPATCHEDGE_CONTINUOUS  = 1,
;   D3DPATCHEDGE_FORCE_DWORD = 0x7fffffff,
;} D3DPATCHEDGESTYLE;
;
;typedef enum _D3DSTATEBLOCKTYPE
;{
;    D3DSBT_ALL           = 1, // capture all state
;    D3DSBT_PIXELSTATE    = 2, // capture pixel state
;    D3DSBT_VERTEXSTATE   = 3, // capture vertex state
;    D3DSBT_FORCE_DWORD   = 0x7fffffff,
;} D3DSTATEBLOCKTYPE;

;// The D3DVERTEXBLENDFLAGS type is used with D3DRS_VERTEXBLEND state.
;//
;typedef enum _D3DVERTEXBLENDFLAGS
;{
;    D3DVBF_DISABLE  = 0,     // Disable vertex blending
;    D3DVBF_1WEIGHTS = 1,     // 2 matrix blending
;    D3DVBF_2WEIGHTS = 2,     // 3 matrix blending
;    D3DVBF_3WEIGHTS = 3,     // 4 matrix blending
;    D3DVBF_TWEENING = 255,   // blending using D3DRS_TWEENFACTOR
;    D3DVBF_0WEIGHTS = 256,   // one matrix is used with weight 1.0
;    D3DVBF_FORCE_DWORD = 0x7fffffff, // force 32-bit size enum
;} D3DVERTEXBLENDFLAGS;

;typedef enum _D3DTEXTURETRANSFORMFLAGS {
;    D3DTTFF_DISABLE         = 0,    // texture coordinates are passed directly
;    D3DTTFF_COUNT1          = 1,    // rasterizer should expect 1-D texture coords
;    D3DTTFF_COUNT2          = 2,    // rasterizer should expect 2-D texture coords
;    D3DTTFF_COUNT3          = 3,    // rasterizer should expect 3-D texture coords
;    D3DTTFF_COUNT4          = 4,    // rasterizer should expect 4-D texture coords
;    D3DTTFF_PROJECTED       = 256,  // texcoords to be divided by COUNTth element
;    D3DTTFF_FORCE_DWORD     = 0x7fffffff,
;} D3DTEXTURETRANSFORMFLAGS;

;// Macros to set texture coordinate format bits in the FVF id

D3DFVF_TEXTUREFORMAT2 equ 0         ;Two floating point values
D3DFVF_TEXTUREFORMAT1 equ 3         ;One floating point value
D3DFVF_TEXTUREFORMAT3 equ 1         ;Three floating point values
D3DFVF_TEXTUREFORMAT4 equ 2         ;Four floating point values

;D3DFVF_TEXCOORDSIZE3(CoordIndex) (D3DFVF_TEXTUREFORMAT3 << (CoordIndex*2 + 16))
;D3DFVF_TEXCOORDSIZE2(CoordIndex) (D3DFVF_TEXTUREFORMAT2)
;D3DFVF_TEXCOORDSIZE4(CoordIndex) (D3DFVF_TEXTUREFORMAT4 << (CoordIndex*2 + 16))
;D3DFVF_TEXCOORDSIZE1(CoordIndex) (D3DFVF_TEXTUREFORMAT1 << (CoordIndex*2 + 16))


;//---------------------------------------------------------------------

;/* Direct3D9 Device types */
;typedef enum _D3DDEVTYPE
;{
    D3DDEVTYPE_HAL         equ 00000001h
    D3DDEVTYPE_REF         equ 00000002h
    D3DDEVTYPE_SW          equ 00000003h
    D3DDEVTYPE_NULLREF     equ 00000004h

;    D3DDEVTYPE_FORCE_DWORD  = 0x7fffffff
;} D3DDEVTYPE;

;/* Multi-Sample buffer types */
;typedef enum _D3DMULTISAMPLE_TYPE
;{
;    D3DMULTISAMPLE_NONE            =  0,
;    D3DMULTISAMPLE_NONMASKABLE     =  1,
;    D3DMULTISAMPLE_2_SAMPLES       =  2,
;    D3DMULTISAMPLE_3_SAMPLES       =  3,
;    D3DMULTISAMPLE_4_SAMPLES       =  4,
;    D3DMULTISAMPLE_5_SAMPLES       =  5,
;    D3DMULTISAMPLE_6_SAMPLES       =  6,
;    D3DMULTISAMPLE_7_SAMPLES       =  7,
;    D3DMULTISAMPLE_8_SAMPLES       =  8,
;    D3DMULTISAMPLE_9_SAMPLES       =  9,
;    D3DMULTISAMPLE_10_SAMPLES      = 10,
;    D3DMULTISAMPLE_11_SAMPLES      = 11,
;    D3DMULTISAMPLE_12_SAMPLES      = 12,
;    D3DMULTISAMPLE_13_SAMPLES      = 13,
;    D3DMULTISAMPLE_14_SAMPLES      = 14,
;    D3DMULTISAMPLE_15_SAMPLES      = 15,
;    D3DMULTISAMPLE_16_SAMPLES      = 16,

;    D3DMULTISAMPLE_FORCE_DWORD     = 0x7fffffff
;} D3DMULTISAMPLE_TYPE;

;/* Formats
; * Most of these names have the following convention:
; *      A = Alpha
; *      R = Red
; *      G = Green
; *      B = Blue
; *      X = Unused Bits
; *      P = Palette
; *      L = Luminance
; *      U = dU coordinate for BumpMap
; *      V = dV coordinate for BumpMap
; *      S = Stencil
; *      D = Depth (e.g. Z or W buffer)
; *      C = Computed from other channels (typically on certain read operations)
; *
; *      Further, the order of the pieces are from MSB first; hence
; *      D3DFMT_A8L8 indicates that the high byte of this two byte
; *      format is alpha.
; *
; *      D3DFMT_D16_LOCKABLE indicates:
; *           - An integer 16-bit value.
; *           - An app-lockable surface.
; *
; *      D3DFMT_D32F_LOCKABLE indicates:
; *           - An IEEE 754 floating-point value.
; *           - An app-lockable surface.
; *
; *      All Depth/Stencil formats except D3DFMT_D16_LOCKABLE and D3DFMT_D32F_LOCKABLE indicate:
; *          - no particular bit ordering per pixel, and
; *          - are not app lockable, and
; *          - the driver is allowed to consume more than the indicated
; *            number of bits per Depth channel (but not Stencil channel).
; */
;#ifndef MAKEFOURCC
;    MAKEFOURCC(ch0, ch1, ch2, ch3)                              \
;                ((DWORD)(BYTE)(ch0) | ((DWORD)(BYTE)(ch1) << 8) |       \
;                ((DWORD)(BYTE)(ch2) << 16) | ((DWORD)(BYTE)(ch3) << 24 ))
;#endif /* defined(MAKEFOURCC) */


;typedef enum _D3DFORMAT
;{
   D3DFMT_UNKNOWN              equ  0

   D3DFMT_R8G8B8               equ 20
   D3DFMT_A8R8G8B8             equ 21
   D3DFMT_X8R8G8B8             equ 22
   D3DFMT_R5G6B5               equ 23
   D3DFMT_X1R5G5B5             equ 24
   D3DFMT_A1R5G5B5             equ 25
   D3DFMT_A4R4G4B4             equ 26
   D3DFMT_R3G3B2               equ 27
   D3DFMT_A8                   equ 28
   D3DFMT_A8R3G3B2             equ 29
   D3DFMT_X4R4G4B4             equ 30
   D3DFMT_A2B10G10R10          equ 31
   D3DFMT_A8B8G8R8             equ 32
   D3DFMT_X8B8G8R8             equ 33
   D3DFMT_G16R16               equ 34
   D3DFMT_A2R10G10B10          equ 35
   D3DFMT_A16B16G16R16         equ 36

   D3DFMT_A8P8                 equ 40
   D3DFMT_P8                   equ 41

   D3DFMT_L8                   equ 50
   D3DFMT_A8L8                 equ 51
   D3DFMT_A4L4                 equ 52

   D3DFMT_V8U8                 equ 60
   D3DFMT_L6V5U5               equ 61
   D3DFMT_X8L8V8U8             equ 62
   D3DFMT_Q8W8V8U8             equ 63
   D3DFMT_V16U16               equ 64
   D3DFMT_A2W10V10U10          equ 67

;   D3DFMT_UYVY                 = MAKEFOURCC('U', 'Y', 'V', 'Y'),
;   D3DFMT_R8G8_B8G8            = MAKEFOURCC('R', 'G', 'B', 'G'),
;   D3DFMT_YUY2                 = MAKEFOURCC('Y', 'U', 'Y', '2'),
;   D3DFMT_G8R8_G8B8            = MAKEFOURCC('G', 'R', 'G', 'B'),
;   D3DFMT_DXT1                 = MAKEFOURCC('D', 'X', 'T', '1'),
;   D3DFMT_DXT2                 = MAKEFOURCC('D', 'X', 'T', '2'),
;   D3DFMT_DXT3                 = MAKEFOURCC('D', 'X', 'T', '3'),
;   D3DFMT_DXT4                 = MAKEFOURCC('D', 'X', 'T', '4'),
;   D3DFMT_DXT5                 = MAKEFOURCC('D', 'X', 'T', '5'),

;   D3DFMT_D16_LOCKABLE         = 70,
;   D3DFMT_D32                  = 71,
;   D3DFMT_D15S1                = 73,
;   D3DFMT_D24S8                = 75,
;   D3DFMT_D24X8                = 77,
;   D3DFMT_D24X4S4              = 79,
;   D3DFMT_D16                  = 80,

;   D3DFMT_D32F_LOCKABLE        = 82,
;   D3DFMT_D24FS8               = 83,


;   D3DFMT_L16                  = 81,

;   D3DFMT_VERTEXDATA           =100,
;   D3DFMT_INDEX16              =101,
;   D3DFMT_INDEX32              =102,

;   D3DFMT_Q16W16V16U16         =110,

;   D3DFMT_MULTI2_ARGB8         = MAKEFOURCC('M','E','T','1'),

;   // Floating point surface formats

;   // s10e5 formats (16-bits per channel)
;   D3DFMT_R16F                 = 111,
;   D3DFMT_G16R16F              = 112,
;   D3DFMT_A16B16G16R16F        = 113,

;   // IEEE s23e8 formats (32-bits per channel)
;   D3DFMT_R32F                 = 114,
;   D3DFMT_G32R32F              = 115,
;   D3DFMT_A32B32G32R32F        = 116,
;
;   D3DFMT_CxV8U8               = 117,
;
;
;   D3DFMT_FORCE_DWORD          =0x7fffffff
; D3DFORMAT;

;/* Display Modes */
D3DDISPLAYMODE STRUCT
    jWidth         UINT   ?
    Height        UINT   ?
    RefreshRate   UINT   ?
    Format        DWORD  ?     ;Was D3DFORMAT
D3DDISPLAYMODE ends

;/* Creation Parameters */
D3DDEVICE_CREATION_PARAMETERS STRUCT
    AdapterOrdinal  UINT    ?
    DeviceType      DWORD   ?       : Was D3DDEVTYPE
    hFocusWindow    HWND    ?
    BehaviorFlags   DWORD   ?
D3DDEVICE_CREATION_PARAMETERS ends


;/* SwapEffects */
;typedef enum _D3DSWAPEFFECT
;{
    D3DSWAPEFFECT_DISCARD           equ 00000001h
    D3DSWAPEFFECT_FLIP              equ 00000002h
    D3DSWAPEFFECT_COPY              equ 00000003h

;    D3DSWAPEFFECT_FORCE_DWORD       = 0x7fffffff
;} D3DSWAPEFFECT;

;/* Pool types */
;typedef enum _D3DPOOL {
    D3DPOOL_DEFAULT                 equ 0
    D3DPOOL_MANAGED                 equ 1
    D3DPOOL_SYSTEMMEM               equ 2
;    D3DPOOL_SCRATCH                 = 3,

;    D3DPOOL_FORCE_DWORD             = 0x7fffffff
;} D3DPOOL;


;/* RefreshRate pre-defines */
D3DPRESENT_RATE_DEFAULT    equ     00000000h


;/* Resize Optional Parameters */
D3DPRESENT_PARAMETERS struct
    BackBufferWidth             UINT            ?
    BackBufferHeight            UINT            ?
    BackBufferFormat            DWORD           ?         ;Was D3DFORMAT
    BackBufferCount             UINT            ?
    MultiSampleType             DWORD           ?         ;Was D3DMULTISAMPLE_TYPE
    MultiSampleQuality          DWORD           ?
    SwapEffect                  DWORD           ?         ;Was D3DSWAPEFFECT
    hDeviceWindow               HWND            ?
    Windowed                    BOOL            ?
    EnableAutoDepthStencil      BOOL            ?
    AutoDepthStencilFormat      DWORD           ?         ;Was  D3DFORMAT
    Flags                       DWORD           ?
    ;FullScreen_RefreshRateInHz must be zero for Windowed mode
    FullScreen_RefreshRateInHz  UINT            ?
    PresentationInterval        UINT            ?
D3DPRESENT_PARAMETERS ends

;Values for D3DPRESENT_PARAMETERS.Flags

D3DPRESENTFLAG_LOCKABLE_BACKBUFFER    equ  00000001h
D3DPRESENTFLAG_DISCARD_DEPTHSTENCIL   equ  00000002h
D3DPRESENTFLAG_DEVICECLIP             equ  00000004h
D3DPRESENTFLAG_VIDEO                  equ  00000010h

;Gamma Ramp: Same as DX7

D3DGAMMARAMP STRUCT
    red   WORD    256
    green WORD    256
    blue  WORD    256
D3DGAMMARAMP ends

;Back buffer types */
;typedef enum _D3DBACKBUFFER_TYPE
;{
;    D3DBACKBUFFER_TYPE_MONO         = 0,
;    D3DBACKBUFFER_TYPE_LEFT         = 1,
;    D3DBACKBUFFER_TYPE_RIGHT        = 2,

;    D3DBACKBUFFER_TYPE_FORCE_DWORD  = 0x7fffffff
;} D3DBACKBUFFER_TYPE;


;/* Types */
;typedef enum _D3DRESOURCETYPE {
;    D3DRTYPE_SURFACE                =  1,
;    D3DRTYPE_VOLUME                 =  2,
;    D3DRTYPE_TEXTURE                =  3,
;    D3DRTYPE_VOLUMETEXTURE          =  4,
;    D3DRTYPE_CUBETEXTURE            =  5,
;    D3DRTYPE_VERTEXBUFFER           =  6,
;    D3DRTYPE_INDEXBUFFER            =  7,


;    D3DRTYPE_FORCE_DWORD            = 0x7fffffff
;} D3DRESOURCETYPE;

;/* Usages */
D3DUSAGE_RENDERTARGET    equ   00000001h
D3DUSAGE_DEPTHSTENCIL    equ   00000002h
D3DUSAGE_DYNAMIC         equ   00000200h


;When passed to CheckDeviceFormat, D3DUSAGE_AUTOGENMIPMAP may return
;D3DOK_NOAUTOGEN if the device doesn't support autogeneration for that format.
;D3DOK_NOAUTOGEN is a success code, not a failure code... the SUCCEEDED and FAILED macros
;will return true and false respectively for this code.
D3DUSAGE_AUTOGENMIPMAP      equ   00000400h
D3DUSAGE_DMAP               equ   00004000h

;The following usages are valid only for querying CheckDeviceFormat
D3DUSAGE_QUERY_LEGACYBUMPMAP            equ   00008000h
D3DUSAGE_QUERY_SRGBREAD                 equ   00010000h
D3DUSAGE_QUERY_FILTER                   equ   00020000h
D3DUSAGE_QUERY_SRGBWRITE                equ   00040000h
D3DUSAGE_QUERY_POSTPIXELSHADER_BLENDING equ   00080000h
D3DUSAGE_QUERY_VERTEXTEXTURE            equ   00100000h
D3DUSAGE_QUERY_WRAPANDMIP	          equ   00200000h

;Usages for Vertex/Index buffers
D3DUSAGE_WRITEONLY          equ   00000008h
D3DUSAGE_SOFTWAREPROCESSING equ   00000010h
D3DUSAGE_DONOTCLIP          equ   00000020h
D3DUSAGE_POINTS             equ   00000040h
D3DUSAGE_RTPATCHES          equ   00000080h
D3DUSAGE_NPATCHES           equ   00000100h

;/* CubeMap Face identifiers */
;typedef enum _D3DCUBEMAP_FACES
;{
;    D3DCUBEMAP_FACE_POSITIVE_X     = 0,
;    D3DCUBEMAP_FACE_NEGATIVE_X     = 1,
;    D3DCUBEMAP_FACE_POSITIVE_Y     = 2,
;    D3DCUBEMAP_FACE_NEGATIVE_Y     = 3,
;    D3DCUBEMAP_FACE_POSITIVE_Z     = 4,
;    D3DCUBEMAP_FACE_NEGATIVE_Z     = 5,

;    D3DCUBEMAP_FACE_FORCE_DWORD    = 0x7fffffff
;} D3DCUBEMAP_FACES;


;Lock flags
D3DLOCK_READONLY           equ   00000010h
D3DLOCK_DISCARD            equ   00002000h
D3DLOCK_NOOVERWRITE        equ   00001000h
D3DLOCK_NOSYSLOCK          equ   00000800h
D3DLOCK_DONOTWAIT          equ   00004000h                 
D3DLOCK_NO_DIRTY_UPDATE    equ   00008000h

;Vertex Buffer Description
D3DVERTEXBUFFER_DESC STRUCT
    Format              DWORD		?	;Was D3DFORMAT
    jType               DWORD       ?     ;Was D3DRESOURCETYPE
    Usage               DWORD       ?
    Pool                DWORD       ?     ;Was D3DPOOL
    jSize               UINT        ?
    FVF                 DWORD       ?
D3DVERTEXBUFFER_DESC ENDS

;Index Buffer Description
D3DINDEXBUFFER_DESC struct
    Format              DWORD ?      ;Was D3DFORMAT
    jType               DWORD ?      ;Was D3DRESOURCETYPE
    Usage               DWORD ?
    Pool                DWORD ?      ;Was D3DPOOL
    jSize               UINT ?
D3DINDEXBUFFER_DESC ends


;Surface Description
D3DSURFACE_DESC struct
    Format                DWORD       ?          ;Was D3DFORMAT
    jType                 DWORD       ?          ;Was D3DRESOURCETYPE
    Usage                 DWORD       ?
    Pool                  DWORD       ?          ;Was D3DPOOL
    MultiSampleType       DWORD       ?          ;Was D3DMULTISAMPLE_TYPE 
    MultiSampleQuality    DWORD       ?
    jWidth                UINT        ?
    Height                UINT        ?
D3DSURFACE_DESC ends

D3DVOLUME_DESC struct
    Format              DWORD          ?         ;Was D3DFORMAT
    jType               DWORD          ?         ;Was D3DRESOURCETYPE
    Usage               DWORD          ?
    Pool                DWORD          ?         ;Was D3DPOOL
    jWidth              UINT           ?
    Height              UINT           ?
    Depth               UINT           ?
D3DVOLUME_DESC ends

;Structure for LockRect
D3DLOCKED_RECT struct
    Pitch               WORD  ?    ;Was INT
    pBits               DWORD ?    ;Was a VOID*
D3DLOCKED_RECT ends

;Structures for LockBox
D3DBOX struct
    Left     UINT    ?
    Top      UINT    ?
    Right    UINT    ?
    Bottom   UINT    ?
    Front    UINT    ?
    Back     UINT    ?
D3DBOX ends

D3DLOCKED_BOX struct
    RowPitch     WORD   ?           ;was INT
    SlicePitch   WORD   ?           ;Was INT
    pBits        DWORD  ?           ;Was void*
D3DLOCKED_BOX ends

;Structures for LockRange
D3DRANGE struct
    jOffset     UINT ?
    jSize        UINT ?
D3DRANGE ends

;Structures for high order primitives
D3DRECTPATCH_INFO struct
    StartVertexOffsetWidth    UINT   ?
    StartVertexOffsetHeight   UINT   ?
    jWidth                     UINT   ?
    Height                    UINT   ?
    Stride                    UINT   ?
    Basis                     DWORD  ?   ;Was D3DBASISTYPE
    Degree                    DWORD  ?   ;Was D3DDEGREETYPE
D3DRECTPATCH_INFO ends

D3DTRIPATCH_INFO struct
     StartVertexOffset    UINT    ?
     NumVertices          UINT    ?
     Basis                DWORD   ?    ;Was D3DBASISTYPE
     Degree               DWORD   ?    ;Was D3DDEGREETYPE
D3DTRIPATCH_INFO ends

;Adapter Identifier

MAX_DEVICE_IDENTIFIER_STRING    equ   512

D3DADAPTER_IDENTIFIER9 struct
    Driver                    byte   MAX_DEVICE_IDENTIFIER_STRING dup (0)
    Description               byte   MAX_DEVICE_IDENTIFIER_STRING dup (0)
    DeviceName                byte   32 dup (0)                            ;Device name for GDI (ex. \\.\DISPLAY1)

    ifdef _WIN32
        DriverVersion LARGE_INTEGER                                            ;Defined for 32 bit components
    else
        DriverVersionLowPart      DWORD  ?                                     ;Defined for 16 bit driver components
        DriverVersionHighPart     DWORD  ?      
    endif

    VendorId                  DWORD        ?
    DeviceId                  DWORD        ?
    SubSysId                  DWORD        ?
    Revision                  DWORD        ?
    DeviceIdentifier          GUID         <>
    WHQLLevel                 DWORD        ?
D3DADAPTER_IDENTIFIER9 ends


;Raster Status structure returned by GetRasterStatus
D3DRASTER_STATUS struct
      InVBlank    BOOL      ?
      ScanLine    UINT      ?
D3DRASTER_STATUS ends

;Debug monitor tokens (DEBUG only)

;Note that if D3DRS_DEBUGMONITORTOKEN is set, the call is treated as
;passing a token to the debug monitor.  For example, if, after passing
;D3DDMT_ENABLE/DISABLE to D3DRS_DEBUGMONITORTOKEN other token values
;are passed in, the enabled/disabled state of the debug
;monitor will still persist.
;The debug monitor defaults to enabled.
;Calling GetRenderState on D3DRS_DEBUGMONITORTOKEN is not of any use.

;typedef enum _D3DDEBUGMONITORTOKENS {
;    D3DDMT_ENABLE            = 0,    // enable debug monitor
;    D3DDMT_DISABLE           = 1,    // disable debug monitor
;    D3DDMT_FORCE_DWORD     = 0x7fffffff,
;} D3DDEBUGMONITORTOKENS;

;// Async feedback

;typedef enum _D3DQUERYTYPE {
;    D3DQUERYTYPE_VCACHE                 = 4, /* D3DISSUE_END */
;    D3DQUERYTYPE_RESOURCEMANAGER        = 5, /* D3DISSUE_END */
;    D3DQUERYTYPE_VERTEXSTATS            = 6, /* D3DISSUE_END */
;    D3DQUERYTYPE_EVENT                  = 8, /* D3DISSUE_END */
;    D3DQUERYTYPE_OCCLUSION              = 9, /* D3DISSUE_BEGIN, D3DISSUE_END */
;    D3DQUERYTYPE_TIMESTAMP              = 10, /* D3DISSUE_END */
;    D3DQUERYTYPE_TIMESTAMPDISJOINT      = 11, /* D3DISSUE_BEGIN, D3DISSUE_END */
;    D3DQUERYTYPE_TIMESTAMPFREQ          = 12, /* D3DISSUE_END */
;    D3DQUERYTYPE_PIPELINETIMINGS        = 13, /* D3DISSUE_BEGIN, D3DISSUE_END */
;    D3DQUERYTYPE_INTERFACETIMINGS       = 14, /* D3DISSUE_BEGIN, D3DISSUE_END */
;    D3DQUERYTYPE_VERTEXTIMINGS          = 15, /* D3DISSUE_BEGIN, D3DISSUE_END */
;    D3DQUERYTYPE_PIXELTIMINGS           = 16, /* D3DISSUE_BEGIN, D3DISSUE_END */
;    D3DQUERYTYPE_BANDWIDTHTIMINGS       = 17, /* D3DISSUE_BEGIN, D3DISSUE_END */
;    D3DQUERYTYPE_CACHEUTILIZATION       = 18, /* D3DISSUE_BEGIN, D3DISSUE_END */
;} D3DQUERYTYPE;

;// Flags field for Issue
;D3DISSUE_END (1 << 0) // Tells the runtime to issue the end of a query, changing it's state to "non-signaled".
;D3DISSUE_BEGIN (1 << 1) // Tells the runtime to issue the beginng of a query.


;// Flags field for GetData
;D3DGETDATA_FLUSH (1 << 0) // Tells the runtime to flush if the query is outstanding.


D3DRESOURCESTATS struct
;Data collected since last Present()
    bThrashing               BOOL   ?      ;indicates if thrashing
    ApproxBytesDownloaded    DWORD  ?      ;Approximate number of bytes downloaded by resource manager
    NumEvicts                DWORD  ?      ;number of objects evicted
    NumVidCreates            DWORD  ?      ;number of objects created in video memory
    LastPri                  DWORD  ?      ;priority of last object evicted
    NumUsed                  DWORD  ?      ;number of objects set to the device
    NumUsedInVidMem          DWORD  ?      ;number of objects set to the device, which are already in video memory
;Persistent data
    WorkingSet               DWORD  ?      ;number of objects in video memory
    WorkingSetBytes          DWORD  ?      ;number of bytes in video memory
    TotalManaged             DWORD  ?      ;total number of managed objects
    TotalBytes               DWORD  ?      ;total number of bytes of managed objects
D3DRESOURCESTATS  ends

;D3DRTYPECOUNT (D3DRTYPE_INDEXBUFFER+1)

;typedef struct _D3DDEVINFO_RESOURCEMANAGER
;{
;    D3DRESOURCESTATS    stats[D3DRTYPECOUNT];
;} D3DDEVINFO_RESOURCEMANAGER, *LPD3DDEVINFO_RESOURCEMANAGER;

D3DDEVINFO_D3DVERTEXSTATS struct
    NumRenderedTriangles       DWORD ?    ;total number of triangles that are not clipped in this frame
    NumExtraClippingTriangles  DWORD ?    ;Number of new triangles generated by clipping
D3DDEVINFO_D3DVERTEXSTATS ends


D3DDEVINFO_VCACHE struct
    Pattern       DWORD  ? ;bit pattern, return value must be FOUR_CC(C, A, C, H) */
    OptMethod     DWORD  ? ;optimization method 0 means longest strips, 1 means vertex cache based */
    CacheSize     DWORD  ? ;cache size to optimize for  (only required if type is 1) */
    MagicNumber   DWORD  ? ;used to determine when to restart strips (only required if type is 1)*/
D3DDEVINFO_VCACHE ends

D3DDEVINFO_D3D9PIPELINETIMINGS struct
     VertexProcessingTimePercent   FLOAT   ?
     PixelProcessingTimePercent    FLOAT   ?
     OtherGPUProcessingTimePercent FLOAT   ?
     GPUIdleTimePercent            FLOAT   ?
D3DDEVINFO_D3D9PIPELINETIMINGS ends

D3DDEVINFO_D3D9INTERFACETIMINGS struct
     WaitingForGPUToUseApplicationResourceTimePercent      FLOAT ?
     WaitingForGPUToAcceptMoreCommandsTimePercent          FLOAT ?
     WaitingForGPUToStayWithinLatencyTimePercent           FLOAT ?
     WaitingForGPUExclusiveResourceTimePercent             FLOAT ?
     WaitingForGPUOtherTimePercent                         FLOAT ?
D3DDEVINFO_D3D9INTERFACETIMINGS ends

D3DDEVINFO_D3D9STAGETIMINGS struct
     MemoryProcessingPercent      FLOAT  ?
     ComputationProcessingPercent FLOAT  ?
D3DDEVINFO_D3D9STAGETIMINGS ends

D3DDEVINFO_D3D9BANDWIDTHTIMINGS struct
     MaxBandwidthUtilized                  FLOAT    ?
     FrontEndUploadMemoryUtilizedPercent   FLOAT    ?
     VertexRateUtilizedPercent             FLOAT    ?
     TriangleSetupRateUtilizedPercent      FLOAT    ?
     FillRateUtilizedPercent               FLOAT    ?
D3DDEVINFO_D3D9BANDWIDTHTIMINGS ends

D3DDEVINFO_D3D9CACHEUTILIZATION struct
     TextureCacheHitRate                   FLOAT    ?  ;Percentage of cache hits
     PostTransformVertexCacheHitRate       FLOAT    ?
D3DDEVINFO_D3D9CACHEUTILIZATION ends


ELSE
echo -----------------------------------------
echo WARNING Duplicate include file d3d9types.inc
echo -----------------------------------------
ENDIF